
-- load libraries
ItemWishlist = LibStub("AceAddon-3.0"):NewAddon("ItemWishlist", "AceConsole-3.0")
local ItemWishlist = ItemWishlist
local AceGUI = LibStub("AceGUI-3.0")


------------------------------------------------------------------------------------------------
-- helper functions

-- Concat the contents of the parameter list,
-- separated by the string delimiter (just like in perl)
-- example: strjoin(", ", {"Anna", "Bob", "Charlie", "Dolores"})
local function strjoin(delimiter, list)
	local len = getn(list)
	if len == 0 then
		return ""
	end
	local string = list[1]
	for i = 2, len do
		string = string .. delimiter .. list[i]
	end
	return string
end

-- Split text into a list consisting of the strings in text,
-- separated by strings matching delimiter (which may be a pattern).
-- example: strsplit(",%s*", "Anna, Bob, Charlie,Dolores")
local function strsplit(delimiter, text)
	local list = {}
	local pos = 1
	if strfind("", delimiter, 1) then -- this would result in endless loops
		error("delimiter matches empty string!")
	end
	while 1 do
		local first, last = strfind(text, delimiter, pos)
		if first then -- found?
			tinsert(list, strsub(text, pos, first-1))
			pos = last+1
		else
			tinsert(list, strsub(text, pos))
			break
		end
	end
	return list
end


-- returns the length of a table with key, value pairs
local function tablelength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end


-- returns an iterator for an table with key, value pairs
local function pairsByKeys (t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
  table.sort(a, f)
  local i = 0      -- iterator variable
  local iter = function ()   -- iterator function
	i = i + 1
	if a[i] == nil then return nil
	else return a[i], t[a[i]]
	end
  end
  return iter
end


------------------------------------------------------------------------------------------------
-- functions

function ItemWishlist:OnInitialize()
	self.inputValue = ""
	self.instanceDifficulty = "0"
	self.mainFrame = nil
	self.itemContainer = nil

	local defaults = {
		profile = {
			itemList = {},
			anchorPoints = {"CENTER", nil,"CENTER", 0, 0},
		}
	}
	self.db = LibStub("AceDB-3.0"):New("ItemWishlistDB",defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "refreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "refreshConfig")
	self.db.RegisterCallback(self, "OnProfileDeleted","refreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "refreshConfig")
	self.options = {
		name = "ItemWishlist",
		handler = ItemWishlist,
		type = 'group',
		args = {},
	}
	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ItemWishlist", self.options, nil)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ItemWishlist", "ItemWishlist")

end

function ItemWishlist:OnEnable()
	if not self.db["profiles"][self.db:GetCurrentProfile()] then
		self.db["profiles"][self.db:GetCurrentProfile()] = {}
		self.db.profiles[self.db:GetCurrentProfile()]["itemList"] = {}
		self.db.profiles[self.db:GetCurrentProfile()]["anchorPoints"] = {"CENTER", nil,"CENTER", 0, 0}
	end
end

local function parseInput(inputValue, instanceDifficulty)
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType,
		itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice =
		GetItemInfo(inputValue)

	if not instanceDifficulty == "0" then
		itemLink = strsplit(":", itemLink)
		itemLink[15] = instanceDifficulty
		itemLink = strjoin(":", itemLink)
	end

	return itemName, itemLink
end

function ItemWishlist:IsOpen()
	return self.mainFrame and self.mainFrame:IsVisible()
end

function ItemWishlist:AddItemToList(itemLink, instanceDifficulty, notes)
print(notes)
	local itemName, newItemLink = parseInput(itemLink, instanceDifficulty)
	local activeProfile = self.db:GetCurrentProfile()
	self.db.profiles[activeProfile]["itemList"][itemName] = {itemLink=newItemLink, notes=notes, instanceDifficulty=instanceDifficulty}

	ItemWishlist:PopulateEntries(activeProfile)
end

------------------------------------------------------------------------------------------------
-- event handler

local function SetTooltip_Hook(self, ...)
	name, link = GameTooltip:GetItem()
	db = LibStub("AceDB-3.0"):New("ItemWishlistDB")
	if db["profiles"][db:GetCurrentProfile()] then

		if db["profiles"][db:GetCurrentProfile()]["itemList"][name] then
			self:AddLine("On Wishlist", 0.4 , 1 , 0.4 , true)
		end
	end
end

hooksecurefunc(GameTooltip, "SetHyperlink", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetBuybackItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetAuctionItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetCompareItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetLootItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetLootRollItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetQuestItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetInventoryItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetGuildBankItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetBagItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetInboxItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetItemByID", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetMerchantItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetSendMailItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetSocketedItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetTradePlayerItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetTradeTargetItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetTransmogrifyItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetUpgradeItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetVoidDepositItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetVoidItem", SetTooltip_Hook)
hooksecurefunc(GameTooltip, "SetVoidWithdrawalItem", SetTooltip_Hook)

function ItemWishlist:SetInputValue(input)
	self.inputValue = input
end

function ItemWishlist:SetInstanceDifficulty(instanceDiff)
	self.instanceDifficulty = instanceDiff
end

local function deleteItem(itemName)
	ItemWishlist.db.profiles[ItemWishlist.db:GetCurrentProfile()]["itemList"][itemName] = nil
	local activeProfile = ItemWishlist.db:GetCurrentProfile()

	ItemWishlist.itemInfoFrame:SetText("")
	GameTooltip:ClearLines()
	GameTooltip:FadeOut()
	ItemWishlist:PopulateEntries(activeProfile)
end

function ItemWishlist:DislpayNotes(widget, event, button)
	ItemWishlist.itemInfoFrame:SetLabel(widget.userdata.itemName)
	ItemWishlist.itemInfoFrame:SetText(ItemWishlist.db.profiles[ItemWishlist.db:GetCurrentProfile()]
														["itemList"]
														[widget.userdata.itemName]
														["notes"])
	ItemWishlist.itemInfoFrame.userdata.itemName = widget.userdata.itemName

end

local function itemLabelOnClick(widget, event, button)
	if button == "RightButton" then
		deleteItem(widget.userdata.itemName)
	else if button == "LeftButton" then
			ItemWishlist:DislpayNotes(widget, event, button)
		end
	end
end

local function onEnter(widget)
	GameTooltip:ClearLines()
	GameTooltip:SetOwner(widget.frame, "ANCHOR_RIGHT")
	GameTooltip:SetHyperlink(widget.userdata.itemLink)
	GameTooltip:Show()
end

local function onLeave()
	GameTooltip:Hide()
end

local function closeWindow(widget)
	local activeProfile = ItemWishlist.db:GetCurrentProfile()
	local point, relativeTo, relativePoint, xoffset, yoffset = widget:GetPoint(1)
	ItemWishlist.db.profiles[activeProfile]["anchorPoints"] = { point, nil, relativePoint, xoffset, yoffset}

	AceGUI:Release(widget)
end

function ItemWishlist:refreshConfig()
	local activeProfile = self.db:GetCurrentProfile()
	ItemWishlist:PopulateEntries(activeProfile)
end

local function saveInfoFrameChanges(widget, notes)
	local activeProfile = ItemWishlist.db:GetCurrentProfile()
	local itemName = widget.userdata.itemName
	ItemWishlist.db.profiles[activeProfile]["itemList"][itemName].notes = notes
end

------------------------------------------------------------------------------------------------
-- Create frame
function ItemWishlist:CreateFrame()
	self.mainFrame = AceGUI:Create("Frame")
	self.mainFrame:SetTitle("Item Wishlist")
	self.mainFrame:SetStatusText("")
	self.mainFrame:SetCallback("OnClose", closeWindow)
	self.mainFrame:SetLayout("Flow")
	self.mainFrame.width = "fill"
	self.mainFrame:Show()
	self.mainFrame:SetWidth(480)
	self.mainFrame:SetHeight(500)

	local anchorPoints = self.db.profiles[self.db:GetCurrentProfile()]["anchorPoints"]
	self.mainFrame:SetPoint(anchorPoints[1], anchorPoints[2], anchorPoints[3],
							anchorPoints[4], anchorPoints[5])

	local addGroup = AceGUI:Create("InlineGroup")
	addGroup:SetTitle("Add Item")
	addGroup:SetLayout("Flow")
	addGroup.width = "fill"

	-- text field item id or item link
	local editbox = AceGUI:Create("EditBox")
	editbox:SetLabel("Insert ItemId or ItemLink:")
	editbox:SetWidth(200)
	editbox:SetCallback("OnEnterPressed", function(widget, event, input) ItemWishlist:SetInputValue(input) end)
	addGroup:AddChild(editbox)


	-- select Instance difficulty
	local difficultyDropDown = AceGUI:Create("Dropdown")
	difficultyDropDown:SetLabel("Instance difficulty")
	difficultyDropDown:SetWidth(120)
	difficultyDropDown:SetList({["0"]= "----", ["1826"] = "normal", ["1726"] = "heroic", ["1727"] = "mythic"},
								{"0","1826", "1726", "1727"})
	difficultyDropDown:SetCallback("OnValueChanged", function(widget, event, key)
									ItemWishlist:SetInstanceDifficulty(key) end)
	addGroup:AddChild(difficultyDropDown)


	-- add item to list button
	local addButton = AceGUI:Create("Button")
	addButton:SetText("Add Item")
	addButton:SetWidth(100)
	addButton:SetCallback("OnClick", function() ItemWishlist:AddItemToList(self.inputValue, self.instanceDifficulty, "") end)
	addGroup:AddChild(addButton)

	self.mainFrame:AddChild(addGroup)

	local listGroup = AceGUI:Create("InlineGroup")
	listGroup:SetFullHeight(true)
	listGroup:SetLayout("Fill")
	listGroup:SetRelativeWidth(0.4)


	local itemScrollFrame = AceGUI:Create("ScrollFrame")
	itemScrollFrame:SetLayout("Flow")
	listGroup:AddChild(itemScrollFrame)

	self.itemContainer = itemScrollFrame
	local activeProfile = self.db:GetCurrentProfile()
	ItemWishlist:PopulateEntries(activeProfile)

	self.mainFrame:AddChild(listGroup)


	self.showGroup = AceGUI:Create("InlineGroup")
	self.showGroup:SetFullHeight(true)
	self.showGroup:SetRelativeWidth(0.6)
	self.showGroup:SetLayout("Fill")


	self.itemInfoFrame = AceGUI:Create("MultiLineEditBox")
	self.itemInfoFrame:SetText("")
	self.itemInfoFrame:SetCallback("OnEnterPressed", function(widget, event, input) saveInfoFrameChanges(widget,input) end)

	self.showGroup:AddChild(self.itemInfoFrame)

	self.mainFrame:AddChild(self.showGroup)
end

function ItemWishlist:PopulateEntries(activeProfile)
	self.itemContainer:ReleaseChildren()

	if self.db.profiles[activeProfile] then
		local itemList = self.db.profiles[activeProfile].itemList
		if itemList and tablelength(itemList) >0 then

			for itemName,itemData in pairsByKeys(self.db.profiles[activeProfile].itemList) do
				local group = AceGUI:Create("SimpleGroup")
				local label = AceGUI:Create("InteractiveLabel")
				label.width = "fill"
				local p, h, f = label.label:GetFont()
				label:SetFont(p, 12, f)
				label:SetHighlight(0.4,0.4,0,0.5)
				label:SetCallback("OnEnter", onEnter)
				label:SetCallback("OnLeave", onLeave)
				label:SetCallback("OnClick", itemLabelOnClick)
				label:SetText(itemData["itemLink"])
				label.userdata.itemLink = itemData["itemLink"]
				label.userdata.itemName = itemName

				group:AddChild(label)
				self.itemContainer:AddChild(group)
			end
		end
	end
end


------------------------------------------------------------------------------------------------
-- slash commands

SHLASH_RELOADUI1 = "/rl"
SlashCmdList.RELOADUI = ReloadUI

-- Slash commands to toggle mainFrame
SLASH_ITEMWISHLIST1, SLASH_ITEMWISHLIST2 = "/iwl", "/itemwishlist"
SlashCmdList.ITEMWISHLIST =  function()
	if not ItemWishlist:IsOpen() then
		ItemWishlist:CreateFrame()
	end
end
